/*
 * Copyright (c) 2002 by Tsuyoshi Iguchi (zinnia@risky-safety.org)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: driver_scroll.c,v 1.2 2002/05/23 12:09:15 zinnia Exp $
 */

#include "flip.h"

#define DRIVER_NAME "scroll"
static int _timeout = 0;

static int timeout(){ return _timeout; }
#define STEP_DEFAULT "8"
#define SPEED_DEFAULT "3"
#define TYPE_DEFAULT "v"
#define STEP2_DEFAULT "80"

void shiftSurface(SDL_Surface* src, SDL_Surface* tmp, int dx, int dy)
{
  SDL_Rect s;
  SDL_Rect d;

  if(dx != 0){
    s.x = s.y = 0;
    s.x = s.y = 0;
    s.w = dx; s.h = src->h;
    d.x = d.y = 0;
    SDL_BlitSurface(src, &s, tmp, &d);

    s.x = dx; s.y = 0;
    s.w = src->w - dx; s.h = src->h;
    d.x = d.y = 0;
    SDL_BlitSurface(src, &s, src, &d);

    s.x = s.y = 0;
    s.w = dx; s.h = src->h;
    d.x = src->w - dx; d.y = 0;
    SDL_BlitSurface(tmp, &s, src, &d);
  } 

  if(dy != 0){
    /* not yet implemented */
  }
}

static void scroll_martial(SDL_Surface* screen, SDL_Surface* bmp)
{
  int x, y;
  int step, step2;
  int speed;
  SDL_Surface* save;

  SDL_Rect src; SDL_Rect dst;
  timebase_context t;
  int ticks, tick_base;


  save = SDL_ConvertSurface(screen, screen->format, screen->flags);

  speed = ABS(atoi(flip_get_property("speed", SPEED_DEFAULT)));
  step = ABS(atoi(flip_get_property("step", STEP_DEFAULT)));
  step2 = ABS(atoi(flip_get_property("step2", STEP2_DEFAULT)));

  tick_base = 1000 * speed * step / bmp->h;


  timebase_init(&t);
  x = 0; y = 0;
  ticks = 0;

  while(1){
    if(x == 0 && y >= screen->h) break;

    shiftSurface(screen, save, step2, 0);
    shiftSurface(bmp, save, step2, 0);

    x += step2;
    x %= screen->w;

    if(y < screen->h){
      src.x = 0; src.y = step;
      src.w = screen->w;
      src.h = screen->h - step;
      
      dst.x = 0; dst.y = 0;
      SDL_BlitSurface(screen, &src, screen, &dst);

      src.x = 0; src.y = y;
      src.w = bmp->w; src.h = step;
      dst.x = 0; dst.y = screen->h - step;
      SDL_BlitSurface(bmp, &src, screen, &dst);

      y += step;
    } else {
      if(x > 0){
	step2 = screen->w - x;
	if(step2 > 10) step2 /= 2;
      } else {
	step2 = 0;
      }
    }
    ticks += tick_base;
    if(timebase_elapse(&t, ticks) >= 0){
      SDL_UpdateRect(screen, 0, 0, 0, 0);
      doEvents();
    } else {
      _timeout++;
    }
  }

  SDL_UpdateRect(screen, 0, 0, 0, 0);
  SDL_FreeSurface(save);
  return;
}

static void scroll_horizontal(SDL_Surface* screen, SDL_Surface* bmp)
{
  int x;
  int step;
  int speed;

  SDL_Rect src; SDL_Rect dst;
  timebase_context t;

  speed = ABS(atoi(flip_get_property("speed", SPEED_DEFAULT)));
  step = ABS(atoi(flip_get_property("step", STEP_DEFAULT)));

  timebase_init(&t);

  for(x = 0; x < bmp->w; x += step){
    src.x = step; src.y = 0;
    src.w = screen->w - step;
    src.h = screen->h;

    dst.x = 0; dst.y = 0;
    SDL_BlitSurface(screen, &src, screen, &dst);

    src.x = x; src.y = 0;
    src.w = step; src.h = bmp->h;
    dst.x = screen->w - step; dst.y = 0;

    SDL_BlitSurface(bmp, &src, screen, &dst);

    if(timebase_elapse(&t, 1000 * speed * x / bmp->h) >= 0){
      SDL_UpdateRect(screen, 0, 0, 0, 0);
      doEvents();
    } else {
      _timeout++;
    }
  }

  SDL_UpdateRect(screen, 0, 0, 0, 0);
  return;
}

static void scroll_vertical(SDL_Surface* screen, SDL_Surface* bmp)
{
  int y;
  int step;
  int speed;

  SDL_Rect src; SDL_Rect dst;
  timebase_context t;

  speed = ABS(atoi(flip_get_property("speed", SPEED_DEFAULT)));
  step = ABS(atoi(flip_get_property("step", STEP_DEFAULT)));

  timebase_init(&t);

  for(y = 0; y < bmp->h; y += step){
    src.x = 0; src.y = step;
    src.w = screen->w;
    src.h = screen->h - step;

    dst.x = 0; dst.y = 0;
    SDL_BlitSurface(screen, &src, screen, &dst);

    src.x = 0; src.y = y;
    src.w = bmp->w; src.h = step;
    dst.x = 0; dst.y = screen->h - step;
    SDL_BlitSurface(bmp, &src, screen, &dst);

    if(timebase_elapse(&t, 1000 * speed * y / bmp->h) >= 0){
      SDL_UpdateRect(screen, 0, 0, 0, 0);
      doEvents();
    } else {
      _timeout++;
    }
  }

  SDL_UpdateRect(screen, 0, 0, 0, 0);
  return;
}

static void do_scroll(SDL_Surface* screen, SDL_Surface* bmp, int type)
{
  switch(type){
  case 'h':
    scroll_horizontal(screen, bmp);
    break;
  case 'v':
    scroll_vertical(screen, bmp);
    break;
  case 'm':
    scroll_martial(screen, bmp);
    break;
  default:
    fprintf(stderr, "%c: unknown type. assume vertical.\n", type);
    scroll_vertical(screen, bmp);
    break;
  }
  return;
}

static void flip(SDL_Surface* screen, SDL_Surface* bmp)
{

  char* s;

  s = flip_get_property("type", TYPE_DEFAULT);
  /* assert */ if(s == NULL) do_scroll(screen, bmp, *(TYPE_DEFAULT));
  do_scroll(screen, bmp, *s);

}

static char* usage()
{
  return DRIVER_NAME ": scrolling effect.\n"
    "  properties:\n"
    "    speed=n: flip in n sec(" SPEED_DEFAULT ")\n"
    "    step=n: scroll every n dots(" STEP_DEFAULT ")\n"
    "    type=[v|h|m]: scroll type [vertical|horizontal|martial-age](" TYPE_DEFAULT ")\n"
    "    step2=n: (type=m only) scroll horizontaly every n dots (" STEP2_DEFAULT ")";
}

driver_vector scroll_vector = {
  DRIVER_NAME,

  flip,
  timeout, 
  usage,
};
