/*
 * Copyright (c) 2002 by Tsuyoshi Iguchi (zinnia@risky-safety.org)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: driver_zoom.c,v 1.2 2002/05/23 12:09:15 zinnia Exp $
 */

#include "flip.h"
#include "SDL_rotozoom.h"

#define DRIVER_NAME "tv"

static int _timeout = 0;
static int timeout(){ return _timeout; }

static void flip(SDL_Surface* screen, SDL_Surface* bmp)
{
  SDL_Surface* tmp = NULL;
  int by, bw, bh;
  SDL_Rect dst;
  SDL_Surface* src;
  int d;

  timebase_context t;
  int counter;

  timebase_init(&t);
  src = SDL_ConvertSurface(screen, screen->format, screen->flags);
  counter = 0;

#define ZOOM_FACTOR 256
  for(d = ZOOM_FACTOR; d >= 1; d /= 2){
    if(tmp != NULL){
      dst.x = 0; dst.y = by;
      dst.w = bw; dst.h = bh;

      SDL_FillRect(screen, &dst, screen->format->colorkey);
      SDL_FreeSurface(tmp);
    }
    tmp = zoomSurface(src, 1.0, (double)d / ZOOM_FACTOR, 1);
    dst.x = 0; dst.y = (screen->h - tmp->h) / 2;
    dst.w = tmp->w; dst.h = tmp->h;
    by = dst.y;
    bh = dst.h;
    bw = dst.w;
    SDL_BlitSurface(tmp, NULL, screen, &dst);
    counter++;
    if(timebase_elapse(&t, counter * 100) >= 0){
      SDL_UpdateRect(screen, 0, 0, 0, 0);
    } else {
      _timeout++;
    }
  }
  SDL_UpdateRect(screen, 0, 0, 0, 0);
  if(tmp != NULL) SDL_FreeSurface(tmp);
  tmp = NULL;
  for(d = 1; d <= ZOOM_FACTOR; d *= 2){
    if(tmp != NULL){
      dst.x = 0; dst.y = by;
      dst.w = bw; dst.h = bh;

      SDL_FillRect(screen, &dst, screen->format->colorkey);
      SDL_FreeSurface(tmp);
    }
    tmp = zoomSurface(bmp, 1.0, (double)d / ZOOM_FACTOR, 1);
    dst.x = 0; dst.y = (screen->h - tmp->h) / 2;
    dst.w = tmp->w; dst.h = tmp->h;
    by = dst.y;
    bh = dst.h;
    bw = dst.w;

    SDL_BlitSurface(tmp, NULL, screen, &dst);
    counter++;
    if(timebase_elapse(&t, counter * 100) >= 0){
      SDL_UpdateRect(screen, 0, 0, 0, 0);
    } else {
      _timeout++;
    }
  }
  SDL_UpdateRect(screen, 0, 0, 0, 0);
  SDL_FreeSurface(src);
 return;
}

static char* usage(FILE* fp)
{
  return DRIVER_NAME ": tv on/off effect\n"
    "  properties:";
}


driver_vector tv_vector = {
  DRIVER_NAME,
  flip,
  timeout,
  usage,
};

