/*
 * Copyright (c) 2002 by Tsuyoshi Iguchi (zinnia@risky-safety.org)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: driver_warp.c,v 1.2 2002/05/23 12:09:15 zinnia Exp $
 */

#include <stdlib.h>
#include "flip.h"
#define DRIVER_NAME "warp"

#define WARP_STEP_DEFAULT "4"
#define WARP_SPEED_DEFAULT "4"

static int _timeout = 0;
static int timeout(){
  return _timeout;
}

static void flip(SDL_Surface* screen, SDL_Surface* bmp)
{
  int warp_step;
  int speed;
  timebase_context t;
  int x, y, y1;
  SDL_Rect src, dst;

  warp_step = ABS(atoi(flip_get_property("step", WARP_STEP_DEFAULT)));
  speed = ABS(atoi(flip_get_property("speed", WARP_SPEED_DEFAULT)));

  timebase_init(&t);
  for(y = 0; y < bmp->h; y += warp_step){
    src.x = 0; src.y = y;
    src.w = bmp->w; src.h = warp_step;

    for(y1 = y; y1 <= bmp->h - warp_step; y1 += warp_step){
      dst.x = 0; dst.y = y1;
      SDL_BlitSurface(bmp, &src, screen, &dst);
    }
    if(timebase_elapse(&t, speed * 1000 * y / (bmp->h)) >= 0){
      SDL_UpdateRect(screen, 0, 0, 0, 0);
      doEvents();
    } else {
      _timeout++;
    }
  }
}


static char* usage(FILE* fp)
{
  return DRIVER_NAME ": warp effect\n"
    "  properties:\n"
    "    speed=n: flip in n sec(" WARP_SPEED_DEFAULT ")\n"
    "    step=n: flip every n dots(" WARP_STEP_DEFAULT ")";
}
driver_vector warp_vector = {
  DRIVER_NAME,
  flip,
  timeout,
  usage,
};

