/*
 * Copyright (c) 2002 by Tsuyoshi Iguchi (zinnia@risky-safety.org)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: property.c,v 1.2 2002/05/23 12:09:15 zinnia Exp $
 */

#include <stdio.h>
#include <string.h>
typedef struct _flip_property flip_property;

/* static */ struct _flip_property {
  char* key;
  char* value;
  flip_property* next;
  flip_property* prev;
};

static flip_property* head;

static flip_property* _get_property(char* key)
{
  flip_property* l;
  if(head == NULL) return NULL;
  for(l = head->next; l != NULL; l = l->next){
    if(strcasecmp(key, l->key) == 0){
      return l;
    }
  }
  return NULL;
}

static void _for_each( void (*func)(flip_property*) )
{
  flip_property* l;
  if(head == NULL) return;
  for(l = head->next; l != NULL; l = l->next){
    func(l);
  }
}

static void _delete_property(flip_property* l)
{
  flip_property* prev, * next;
  if(l == NULL) return;
  prev = l->prev;
  next = l->next;

  prev->next = next;
  if(next != NULL) next->prev = prev;

  free(l->key);
  free(l->value);
  free(l);
}

static void _print_property(flip_property* l)
{
  fprintf(stderr, "%s: %s\n", l->key, l->value);
}

char* flip_get_property(char* key, char* defaultvalue)
{
  flip_property* l;
  l = _get_property(key);
  if(l == NULL) return defaultvalue;
  return l->value;
}

void flip_add_property(char* key, char* value)
{
  flip_property* p;
  flip_property* tmp = (flip_property*)malloc(sizeof(flip_property));
  //  fprintf(stderr, "%s: %s\n", key, value);
  if(head == NULL){
    head = (flip_property*)malloc(sizeof(flip_property));
    head->next = NULL;
    head->prev = NULL;
  }
  tmp->key = key;
  tmp->value = value;

  p = head->next;
  head->next = tmp;
  tmp->prev = head;

  tmp->next = p;
  if(p != NULL) p->prev = tmp;
}

void flip_add_expression(char* e)
{
  char* key; char* value; char* p;
  key = strdup(e);
  value = NULL;
  for(p = key; *p != '\0'; p++){
    if(*p == '='){
      *p = '\0';
      value = p + 1;
    }
  }
  if(value == NULL) return;
  flip_add_property(key, value);
}

void flip_delete_property(char* key)
{
  _delete_property(_get_property(key));
}

void flip_delete_property_all(){
  _for_each(_delete_property);
}

void flip_print_properties()
{
  _for_each(_print_property);
}
